// Card.h: interface for the CCard class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CARD_H__73211E66_B695_465F_86B6_0F9718691273__INCLUDED_)
#define AFX_CARD_H__73211E66_B695_465F_86B6_0F9718691273__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CTemplate;

class CCard 
{
private:
	CString m_strWord[MAX_ITEM];
	DWORD m_dwMemorizingRate;
	bool IsNeedUpdate(UINT nIndex, LPCTSTR szNewWord) const;
public:
	CCard(DWORD dwMemorizingRate = 0);

	LPCTSTR GetWord(UINT nIndex) const
	{
		ASSERT(nIndex < MAX_ITEM);
		return m_strWord[nIndex];
	}
	bool SetWord(UINT nIndex, LPCTSTR szWord);

	CString GetParsedWord(UINT nIndex, UINT nParseIndex) const;	// [ȯ]  Լ Ʈ о ,,   ش ȣ ´ ܾ ȯѴ
	int GetParsedWordCount(UINT nIndex) const;	// [ȯ]  Լ Ʈ   ,, е ܾ ִ   ȯѴ.
	void Memorized()
	{
		m_dwMemorizingRate = 1;
	}
	bool IsMemorized() const
	{
		return (m_dwMemorizingRate > 0);
	}
	bool IsNotMemorized() const
	{
		return (m_dwMemorizingRate == 0);
	}
	DWORD GetMemorizingRate() const
	{
		return m_dwMemorizingRate;
	}
	void Reset()
	{
		m_dwMemorizingRate = 0;
	}
	bool IsIncludeWord(LPCTSTR szWord) const;
	bool IsIncludeWord(LPCTSTR szWord, UINT uiItemIndex) const;
	void SaveToFile(CFile* pFile) const;
	void SaveToFile(HANDLE hFile) const;
};

typedef CList <CCard*, CCard*> CARD_LIST;

#define MAX_TITLE_LEN 32

class CCardSet 
{
private:

	CARD_LIST m_lstCard; //Master Ʈ
	CARD_LIST m_lstRandomCard; 

	POSITION m_posList; 
	POSITION m_posRandomList;
	POSITION m_posBookmark;
	int m_nMemorized;
	int m_nCardIndex;
	bool m_bModified;

	void MakeRadomCardList();
	int GetIndexFromPosition(POSITION posFind) const;
public:

	CCardSet();
	virtual ~CCardSet();

	UINT GetNumCards() const
	{
		return m_lstCard.GetCount();
	}
	UINT GetNumMemorizedCards() const
	{
		return m_nMemorized;
	}
	UINT GetNumNotMemorizedCards() const
	{
		return m_lstCard.GetCount() - m_nMemorized;
	}
	void IncreaseMemorizedCardCount()
	{
		m_nMemorized++;
		m_bModified = true;
	}
	void DecreaseMemorizedCardCount()
	{
		if( m_nMemorized > 0 )
			m_nMemorized--;
		m_bModified = true;
	}
	int GetCardIndex() const
	{
		return m_nCardIndex;
	}

	//Bookmark
	POSITION GetBookmarkPos() const 
	{
		return m_posBookmark;
	}
	void SetBookmark()
	{
		m_bModified = TRUE;
		m_posBookmark = m_posList;
	}
	void SetBookmark(int nIndex);
	int GetBookmarkIndex() const;

	int GetCurrentPosIndex() const;
	void SetCurrentPos(int nIndex);

	void AddCardToTail(CCard* pCard);
	void InsertCard(CCard* pCard);
	void DeleteCurrentCard();
	void ResetMemorizing();

	void MoveToFirst();
	void MoveToLast();
	bool MoveNext();
	bool MovePrev();
	bool IsLastCard() const;
	bool IsFirstCard() const;
	CCard* GetCard() const;
	CCard* GetRandomCard() const;
	CCard* GetCardFromIndex(int nIndex) const;
	CCard* GetCardFromList(bool bMemorized);
	CCard* GetCardFromRandomList(bool bMemorized);
	void MoveToPosition(POSITION posCard);
	void MoveToIndex(int nIndex);

	void ModifiedCard()
	{
		m_bModified = true;
	}
	bool IsModified() const { return m_bModified; }

	bool ImportTextFile(LPCTSTR szFileName);
	bool ProcessStringLine(CString& strLine);

	void SaveToFile(CFile* pFile) const;
	void SaveToFile(HANDLE hFile) const;
	bool SaveToTextFile(CFile* pFile, bool bUnicode) const;
	bool LoadFromFile(CFile* pFile, DWORD dwNumCards);
	
	POSITION FindCard(LPCTSTR szFind, bool bSelectItemOnly, UINT nSelectItem, bool bMove);
};

#endif // !defined(AFX_CARD_H__73211E66_B695_465F_86B6_0F9718691273__INCLUDED_)
