// CardFile.cpp: implementation of the CCardFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MemoKing.h"
#include "define.h"
#include "CardFile.h"
#include "Card.h"
#include "Template.h"
#include "Quiz.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCardFile::CCardFile()
{

}

CCardFile::~CCardFile()
{

}

bool CCardFile::SaveFile(LPCTSTR szFileName, const CCardSet* pCardSet, const CTemplate* pTemplate, const CQuizSetting* pQuizSetting)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);
	ASSERT(pTemplate != NULL);
	ASSERT(pQuizSetting != NULL);

	CFile fileCardSet;
	CARDSET_HEADER Header;
	CARDSET_HEADER_EX HeaderEx;

	TRY
	{
		if (fileCardSet.Open( szFileName, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive) == FALSE)
			return false;

		Header.wType = MK_FILE_TYPE; //"mk"
		Header.wVersion = MK_FILE_VER;
		Header.dwNumCards = pCardSet->GetNumCards();


		HeaderEx.nBookmark = pCardSet->GetBookmarkIndex();
		HeaderEx.nCurPos = pCardSet->GetCurrentPosIndex();
		HeaderEx.dwReserved1 = 0;
		HeaderEx.dwReserved2 = 0;

		fileCardSet.Write((void*)&Header, sizeof(CARDSET_HEADER));
		fileCardSet.Write((void*)&HeaderEx, sizeof(CARDSET_HEADER_EX));

		pCardSet->SaveToFile(&fileCardSet);
		pTemplate->SaveToFile(&fileCardSet);
		pQuizSetting->SaveToFile(&fileCardSet);

		fileCardSet.Flush();
		fileCardSet.Close();
	}
	CATCH (CFileException, e)
	{
		MessageBox(NULL, _T("  ߻"), _T("ϱ"), MB_OK);
		e->Delete();
	}
	END_CATCH

	return true;
}

bool CCardFile::SaveFile2(LPCTSTR szFileName, const CCardSet* pCardSet, const CTemplate* pTemplate, const CQuizSetting* pQuizSetting)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);
	ASSERT(pTemplate != NULL);
	ASSERT(pQuizSetting != NULL);

	CARDSET_HEADER Header;
	CARDSET_HEADER_EX HeaderEx;

	HANDLE hFile;

	hFile = CreateFile(szFileName, GENERIC_WRITE, 0, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH, NULL);

	if (hFile == INVALID_HANDLE_VALUE)
		return FALSE;

	Header.wType = MK_FILE_TYPE; //"mk"
	Header.wVersion = MK_FILE_VER;
	Header.dwNumCards = pCardSet->GetNumCards();
	HeaderEx.nBookmark = pCardSet->GetBookmarkIndex();
	HeaderEx.nCurPos = pCardSet->GetCurrentPosIndex();
	HeaderEx.dwReserved1 = 0;
	HeaderEx.dwReserved2 = 0;

	DWORD dwWritten;
	WriteFile(hFile, (void*)&Header, sizeof(CARDSET_HEADER), &dwWritten, NULL);
	WriteFile(hFile, (void*)&HeaderEx, sizeof(CARDSET_HEADER_EX), &dwWritten, NULL);
	//fileCardSet.Write((void*)&Header, sizeof(CARDSET_HEADER));
	//fileCardSet.Write((void*)&HeaderEx, sizeof(CARDSET_HEADER_EX));

	pCardSet->SaveToFile(hFile);
	pTemplate->SaveToFile(hFile);
	pQuizSetting->SaveToFile(hFile);

	//fileCardSet.Flush();
	//fileCardSet.Close();
	CloseHandle(hFile);

	return true;
}

bool CCardFile::SaveHeader(LPCTSTR szFileName, const CCardSet* pCardSet)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);

	CFile fileCardSet;
	CARDSET_HEADER Header;
	CARDSET_HEADER_EX HeaderEx;

	TRY 
	{
		if (fileCardSet.Open( szFileName, CFile::modeWrite | CFile::shareExclusive) == FALSE)
			return false;

		Header.wType = MK_FILE_TYPE; //"mk"
		Header.wVersion = MK_FILE_VER;
		Header.dwNumCards = pCardSet->GetNumCards();

		HeaderEx.nBookmark = pCardSet->GetBookmarkIndex();
		HeaderEx.nCurPos = pCardSet->GetCurrentPosIndex();
		HeaderEx.dwReserved1 = 0;
		HeaderEx.dwReserved2 = 0;

		fileCardSet.Write((void*)&Header, sizeof(CARDSET_HEADER));
		fileCardSet.Write((void*)&HeaderEx, sizeof(CARDSET_HEADER_EX));

		fileCardSet.Close();
	}
	CATCH (CFileException, e)
	{
		MessageBox(NULL, _T("  ߻"), _T("׽Ʈ"), MB_OK);
		e->Delete();
	}
	END_CATCH

	return true;
}


bool CCardFile::LoadFile(LPCTSTR szFileName, CCardSet* pCardSet, CTemplate* pTemplate, CQuizSetting* pQuizSetting)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);
	ASSERT(pTemplate != NULL);
	ASSERT(pQuizSetting != NULL);

	CFile fileCardSet;
	CARDSET_HEADER Header;
	CARDSET_HEADER_EX HeaderEx;
	DWORD dwRead;

	TRY
	{
		if (fileCardSet.Open( szFileName, CFile::modeRead | CFile::shareExclusive) == FALSE)
		{
			return false;
		}

		dwRead = fileCardSet.Read((void*)&Header, sizeof(CARDSET_HEADER));
		if (dwRead != sizeof(CARDSET_HEADER))
		{
			// б 
			TRACE(_T("File Read Failed. dwRead != sizeof(CARDSET_HEADER)\n"));
			fileCardSet.Close();
			return false;
		}

		if (Header.wType != MK_FILE_TYPE)
		{
			//  Ʋ 
			TRACE(_T("File Read Failed. Header.wType != MK_FILE_TYPE\n"));
			fileCardSet.Close();
			return false;
		}

		if (Header.wVersion > MK_FILE_VER) // α׷ ϴ  ū  
		{
			TRACE(_T("File Read Failed.  Header.wVersion > MK_FILE_VER \n"));
			fileCardSet.Close();
			return false;
		}

		if (Header.wVersion >= 0x0003) //Ver 3 ̸̻ HeaderEx оδ.
		{
			dwRead = fileCardSet.Read((void*)&HeaderEx, sizeof(CARDSET_HEADER_EX));
			if (dwRead != sizeof(CARDSET_HEADER_EX))
			{
				TRACE(_T("File Read Failed. dwRead != sizeof(CARDSET_HEADER_EX)\n"));
				fileCardSet.Close();
				return false;
			}
		}

		if (pCardSet->LoadFromFile(&fileCardSet, Header.dwNumCards) == false)
		{
			TRACE(_T("pCardSet->LoadFromFile Failed. \n"));
			fileCardSet.Close();
			return false;
		}

		if (pTemplate->LoadFromFile(&fileCardSet, Header.wVersion) == false)
		{
			TRACE(_T("pTemplate->LoadFromFile Failed.\n"));
			fileCardSet.Close();
			return false;
		}

		if (pQuizSetting->LoadFromFile(&fileCardSet, Header.wVersion, Header.dwNumCards) == false)
		{
			TRACE(_T("pQuizSetting->LoadFromFile Failed\n"));
			fileCardSet.Close();
			return false;
		}
		fileCardSet.Close();
	}
	
	CATCH (CFileException, e)
	{
		MessageBox(NULL, _T("Ϸε  ߻"), _T("׽Ʈ"), MB_OK);
		e->Delete();
	}
	END_CATCH

	if (Header.wVersion < MK_FILE_VER) //     ϱ Ͽ 
		pCardSet->ModifiedCard();

	if (Header.wVersion >= 0x0003 ) 
	{
		if (HeaderEx.nBookmark >= 0)
			pCardSet->SetBookmark(HeaderEx.nBookmark);
		pCardSet->SetCurrentPos(HeaderEx.nCurPos);
	}
	else
		pCardSet->MoveToFirst();

	return true;
}

bool CCardFile::ImportTextFile(LPCTSTR szFileName, CCardSet* pCardSet)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);

	//0xFEFF
	//Check Unicode
	CFile fileText;
	if (fileText.Open( szFileName, CFile::modeRead) == FALSE)
		return false;

	UINT uiRead;
	BYTE pBuffer[2];
	const UINT BUFFER_SIZE = 2;
	uiRead = fileText.Read(pBuffer, BUFFER_SIZE);
	fileText.Close();

	CString strSep;
	if (pBuffer[0] == 0xFF && pBuffer[1] == 0xFE)
	{
		if (CheckSeperator(szFileName, true, strSep) == false)
			return false;
		return ImportUnicodeTextFile(szFileName, pCardSet, strSep);
	}
	else
	{
		if (CheckSeperator(szFileName, false, strSep) == false)
			return false;
		return ImportAsciiTextFile(szFileName, pCardSet, strSep);
	}
}

bool CCardFile::ImportAsciiTextFile(LPCTSTR szFileName, CCardSet* pCardSet, LPCTSTR szSep)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);

	CFile fileText;
	if (fileText.Open( szFileName, CFile::modeRead) == FALSE)
		return false;

	const UINT BUFFER_SIZE = 1024;
	char pBuffer[BUFFER_SIZE];
	UINT uiRead;
	CString strLine;
	UINT uiIndex;
	bool bExtract;
	bool bEOF = false;


	do
	{
		memset( pBuffer, 0, BUFFER_SIZE );
		uiRead = fileText.Read(pBuffer, BUFFER_SIZE);
		if (uiRead < BUFFER_SIZE)
			bEOF = true;

		uiIndex = 0;
		char* pTempBuf = pBuffer;
		do 
		{
			bExtract = ExtractLine(pTempBuf, uiIndex, bEOF, strLine);
			if (bExtract == true)
				ProcessStringLine(pCardSet, strLine, szSep);
			if (bEOF == true && uiIndex == uiRead) //ǳ
				break;
		} while(bExtract == true);

		if (uiIndex < uiRead ) // ó   
		{
			// ͸ ׸ŭ ڷ .
			int nLength = uiRead - uiIndex;
			fileText.Seek(-nLength, CFile::current);
		}
	} while(bEOF == false);
	return true;
}

bool CCardFile::ImportUnicodeTextFile(LPCTSTR szFileName, CCardSet* pCardSet, LPCTSTR szSep)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);

	CFile fileText;
	if (fileText.Open( szFileName, CFile::modeRead) == FALSE)
		return false;

	fileText.Seek(2, CFile::begin);

	const UINT BUFFER_SIZE = 1024;
	char pBuffer[BUFFER_SIZE];
	UINT uiRead;
	CString strLine;
	UINT uiIndex;
	bool bExtract;
	bool bEOF = false;

	do
	{
		memset( pBuffer, 0, BUFFER_SIZE );
		uiRead = fileText.Read(pBuffer, BUFFER_SIZE);
		if (uiRead < BUFFER_SIZE)
			bEOF = true;

		uiIndex = 0;
		char* pTempBuf = pBuffer;
		do 
		{
			bExtract = ExtractLineU(pTempBuf, uiIndex, bEOF, strLine);
			if (bExtract == true)
				ProcessStringLine(pCardSet, strLine, szSep);
			if (bEOF == true && uiIndex == uiRead) //ǳ
				break;
		} while( bExtract == true  );

		if (uiIndex < uiRead ) // ó   
		{
			// ͸ ׸ŭ ڷ .
			int nLength = uiRead - uiIndex;
			fileText.Seek(-nLength, CFile::current);
			//uiRead -= nLength;
		}
	} while (bEOF == false);
	return true;
}

bool CCardFile::CheckSeperator(LPCTSTR szFileName, bool bUnicode, CString& strSep)
{
	ASSERT(szFileName != NULL);

	CFile fileText;
	if (fileText.Open( szFileName, CFile::modeRead) == FALSE)
		return false;
	
	if (bUnicode == true)
		fileText.Seek(2, CFile::begin);

	const UINT BUFFER_SIZE = 1024;
	char pBuffer[BUFFER_SIZE];

	memset( pBuffer, 0, BUFFER_SIZE );
	UINT uiRead = fileText.Read(pBuffer, BUFFER_SIZE);

	CString strLine;
	UINT uiIndex = 0;
	bool bEOF = false;
	bool bExtract;

	do
	{
		if (bUnicode == true)
			bExtract = ExtractLineU(pBuffer, uiIndex, bEOF, strLine);
		else
			bExtract = ExtractLine(pBuffer, uiIndex, bEOF, strLine);

		if (strLine.GetAt(0) == _T('#')) //ڸƮ 
			bExtract = false;
	} while (bExtract == false);

	if (bExtract == true)
	{
		if (strLine.Find(_T("//")) >= 0)
		{
			strSep = _T("//");
			return true;
		}
		else if (strLine.Find(_T("\t")) >= 0)
		{
			strSep = _T("\t");
			return true;
		}
		else if (strLine.Find(_T(",")) >= 0)
		{
			strSep = _T(",");
			return true;
		}
		else
			return false;
	}
	
	return false;
}

bool CCardFile::ProcessStringLine(CCardSet* pCardSet, CString& strLine, LPCTSTR szSep)
{
	CCard* pCard;
	int nStart, nEnd;
	CString strWord;

	strLine.TrimLeft();
	strLine.TrimRight();

	int nLenSep = lstrlen(szSep);

	if (strLine.GetLength() > 0)
	{
		if (strLine.GetAt(0) == _T('#')) //ڸƮ 
			return true;
		nStart = 0;
		nEnd = strLine.Find(szSep, nStart);
		pCard = new CCard();
		
		for (int i = 0; i < MAX_ITEM; i++)
		{
			if (nEnd == -1)
			{
				strWord = strLine.Mid(nStart);
				strWord.TrimLeft();
				strWord.TrimRight();
				pCard->SetWord(i, strWord);
				break;
			}
			else
			{
				strWord = strLine.Mid(nStart, (nEnd - nStart));
				strWord.TrimLeft();
				strWord.TrimRight();
				pCard->SetWord(i, strWord );
				nStart = nEnd + nLenSep;
				nEnd = strLine.Find(szSep, nStart);
			}
		}
		pCardSet->AddCardToTail(pCard);
	}
	return true;
}

//ۿ  ξ  CString ش.
//uiIndex      ġ ٲپش.
//Buffer ̻ CR/LF   false return Ѵ.
bool CCardFile::ExtractLine(char* pBuffer, UINT& uiIndex, bool bEOF, CString& strLine)
{
	ASSERT(pBuffer != NULL);
	ASSERT(IsBadReadPtr(pBuffer, (uiIndex + 1 )* sizeof(char)) == FALSE);

	pBuffer = (char*)(pBuffer + uiIndex);
	char* pDest = strstr(pBuffer, "\r\n");

	if (pDest == NULL) //ã  
	{
		if (bEOF == true)
		{
			strLine = CString((LPCSTR)pBuffer);
			uiIndex += strlen(pBuffer);
			return true;
		}
		else
			return false;
	}
	else
	{
		*(pDest) = '\0';
		*(pDest + 1) = '\0';
		strLine = CString((LPCSTR) pBuffer);
		uiIndex += (pDest - pBuffer) + 2;
		//uiIndex += strLine.GetLength() + 2 * sizeof(char);
		return true;
	}
}

bool CCardFile::ExtractLineU(char* pBuffer, UINT& uiIndex, bool bEOF, CString& strLine)
{
	ASSERT(pBuffer != NULL);
	ASSERT(IsBadReadPtr(pBuffer, (uiIndex + 1 )* sizeof(char)) == FALSE);

	pBuffer = (char*)(pBuffer + uiIndex);
	wchar_t* pDest = wcsstr((const wchar_t*)pBuffer, _T("\r\n"));

	if (pDest == NULL) //ã  
	{
		if (bEOF == true)
		{
			strLine = CString((const wchar_t*)pBuffer);
			uiIndex += strLine.GetLength() * sizeof(wchar_t);
			return true;
		}
		else
			return false;
	}
	else
	{
		*(pDest) = _T('\0');
		*(pDest + 1) = _T('\0');
		strLine = CString((const wchar_t*)pBuffer);
		uiIndex += (strLine.GetLength() + 2 ) * sizeof(wchar_t);
		return true;
	}
}

bool CCardFile::ExportToTextFile(LPCTSTR szFileName, CCardSet* pCardSet, bool bUnicode)
{
	ASSERT(szFileName != NULL);
	ASSERT(pCardSet != NULL);

	CFile fileCardSet;
	if (fileCardSet.Open( szFileName, CFile::modeCreate | CFile::modeWrite) == FALSE)
		return false;

	pCardSet->SaveToTextFile(&fileCardSet, bUnicode);

	fileCardSet.Close();
	return true;
}

