// IconButton.cpp : implementation file
//

#include "stdafx.h"
#include "MemoKing.h"
#include "IconButton.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BS_TYPEMASK 0x0000001FL		//SS_TYPEMASK

/////////////////////////////////////////////////////////////////////////////
// CIconButton

CIconButton::CIconButton()
{
	m_hIcon = NULL;
	m_dwWidth = 0;
	m_dwHeight = 0;

	m_byAlign = ST_ALIGN_HORIZ; 

	SetDefaultColors();
}

CIconButton::~CIconButton()
{
	if (m_hIcon != NULL)	
		::DestroyIcon(m_hIcon);
}

DWORD CIconButton::SetDefaultColors()
{
	m_crBtnFace = ::GetSysColor(COLOR_BTNFACE);
	m_crBtnText = ::GetSysColor(COLOR_BTNTEXT);
	return BTNST_OK;
} 

DWORD CIconButton::SetAlign(BYTE byAlign, BOOL bRepaint)
{
	switch (byAlign)
	{    
		case ST_ALIGN_HORIZ:
		case ST_ALIGN_HORIZ_RIGHT:
		case ST_ALIGN_VERT:
			m_byAlign = byAlign;
			if (bRepaint)	Invalidate();
			return BTNST_OK;
			break;
	} // switch

	return BTNST_INVALIDALIGN;
} // End of SetAlign



BEGIN_MESSAGE_MAP(CIconButton, CButton)
	//{{AFX_MSG_MAP(CIconButton)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIconButton message handlers


DWORD CIconButton::SetIcon(int nIconIn, CSize sizeIn)
{
	HICON hIconIn = NULL;
	HINSTANCE hInstResource	= NULL;

	// Find correct resource handle
	hInstResource = AfxFindResourceHandle(MAKEINTRESOURCE(nIconIn), RT_GROUP_ICON);

	// Set icon when the mouse is IN the button
	hIconIn = (HICON)::LoadImage(hInstResource, MAKEINTRESOURCE(nIconIn), IMAGE_ICON, 0, 0, 0);
	return SetIcon(hIconIn, sizeIn);
} // End of SetIcon

DWORD CIconButton::SetIcon(HICON hIconIn, CSize sizeIn)
{
	// Free any loaded resource
	if (m_hIcon != NULL)	
		::DestroyIcon(m_hIcon);

	if (hIconIn != NULL)
	{
		// Icon when mouse over button?
		m_hIcon = hIconIn;
		// Get icon dimension
		m_dwWidth	= (DWORD)sizeIn.cx;
		m_dwHeight	= (DWORD)sizeIn.cy;
	} // if

	Invalidate();

	return BTNST_OK;
} // End of SetIcon

void CIconButton::DrawItem(LPDRAWITEMSTRUCT lpDIS) 
{
	CDC* pDC = CDC::FromHandle(lpDIS->hDC);

	CRect itemRect = lpDIS->rcItem;

	pDC->SetBkMode(TRANSPARENT);

	// Prepare draw... paint button background
	OnDrawBackground(pDC, &itemRect);
	OnDrawBorder(pDC, &itemRect);

	// Read the button's title
	TCHAR szTitle[256];
	GetWindowText(szTitle, sizeof(szTitle)/sizeof(TCHAR));

	CRect captionRect = lpDIS->rcItem;

	// Draw the icon
	if (m_hIcon)
		DrawTheIcon(pDC, _tcslen(szTitle), &lpDIS->rcItem, &captionRect);

	// Write the button title (if any)
	if (_tcslen(szTitle))
	{
   		// Center text
		CRect centerRect = captionRect;
		pDC->DrawText(szTitle, -1, captionRect, DT_WORDBREAK | DT_CENTER | DT_CALCRECT);
		captionRect.OffsetRect((centerRect.Width() - captionRect.Width())/2, (centerRect.Height() - captionRect.Height())/2);

		pDC->SetBkMode(TRANSPARENT);

		pDC->SetTextColor(m_crBtnFace);
		pDC->SetBkColor(m_crBtnText);

		pDC->DrawText(szTitle, -1, captionRect, DT_WORDBREAK | DT_CENTER);
	} // if
}

DWORD CIconButton::OnDrawBorder(CDC* pDC, LPCRECT pRect)
{
	CPen penBlack;
	penBlack.CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	CPen* pOldPen = pDC->SelectObject(&penBlack);

	pDC->MoveTo(pRect->left, pRect->top);
	pDC->LineTo(pRect->right -1, pRect->top);
	pDC->LineTo(pRect->right -1, pRect->bottom -1);
	pDC->LineTo(pRect->left, pRect->bottom - 1);
	pDC->LineTo(pRect->left, pRect->top);

	pDC->SelectObject(pOldPen);
	return BTNST_OK;

}

DWORD CIconButton::OnDrawBackground(CDC* pDC, LPCRECT pRect)
{
	CBrush brBackground(m_crBtnFace);
	pDC->FillRect(pRect, &brBackground);
	return BTNST_OK;
}

void CIconButton::DrawTheIcon(CDC* pDC, BOOL bHasTitle, RECT* rpItem, CRect* rpTitle)
{
	CRect rImage;
	PrepareImageRect(bHasTitle, rpItem, rpTitle, m_dwWidth, m_dwHeight, &rImage);

	pDC->DrawIcon(rImage.TopLeft(), m_hIcon);
} // End of DrawTheIcon

void CIconButton::PrepareImageRect(BOOL bHasTitle, RECT* rpItem, CRect* rpTitle, DWORD dwWidth, DWORD dwHeight, CRect* rpImage)
{
	CRect rBtn;

	rpImage->CopyRect(rpItem);

	switch (m_byAlign)
	{
		case ST_ALIGN_HORIZ:
			if (bHasTitle == FALSE)
			{
				// Center image horizontally
				rpImage->left += ((rpImage->Width() - dwWidth)/2);
			}
			else
			{
				// Image must be placed just inside the focus rect
				rpImage->left += 3;  
				rpTitle->left += dwWidth + 3;
			}
			// Center image vertically
			rpImage->top += ((rpImage->Height() - dwHeight)/2);
			break;

		case ST_ALIGN_HORIZ_RIGHT:
			GetClientRect(&rBtn);
			if (bHasTitle == FALSE)
			{
				// Center image horizontally
				rpImage->left += ((rpImage->Width() - dwWidth)/2);
			}
			else
			{
				// Image must be placed just inside the focus rect
				rpTitle->right = rpTitle->Width() - dwWidth - 3;
				rpTitle->left = 3;
				rpImage->left = rBtn.right - dwWidth - 3;
				// Center image vertically
				rpImage->top += ((rpImage->Height() - dwHeight)/2);
			}
			break;
		
		case ST_ALIGN_VERT:
			// Center image horizontally
			rpImage->left += ((rpImage->Width() - dwWidth)/2);
			if (bHasTitle == FALSE)
			{
				// Center image vertically
				rpImage->top += ((rpImage->Height() - dwHeight)/2);           
			}
			else
			{
				rpImage->top = 3;
				rpTitle->top += dwHeight;
			}
			break;
	}
} // End of PrepareImageRect

void CIconButton::PreSubclassWindow() 
{
	ModifyStyle(BS_TYPEMASK, BS_OWNERDRAW, SWP_FRAMECHANGED);
	
	CButton::PreSubclassWindow();
}
