// Stroke.cpp: implementation of the CStroke class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "define.h"
#include "MemoKing.h"
#include "Stroke.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStroke::CStroke()
{

}

CStroke::~CStroke()
{

}

void CStroke::AddPoint(CPoint* pPoint)
{
	ASSERT(pPoint != NULL);

	CPoint point(pPoint->x, pPoint->y);

	m_arPoint.Add(point);
}

void CStroke::Draw(CDC* pDC) const
{
	pDC->MoveTo(m_arPoint[0]);
	for (int i=1; i < m_arPoint.GetSize(); i++)
	{
		pDC->LineTo(m_arPoint[i]);
	}
}

void CStroke::Finish()
{
	if (m_arPoint.GetSize()==0)
	{
		m_rcBound.SetRectEmpty();
		return;
	}

	CPoint pt = m_arPoint[0];
	m_rcBound = CRect(pt.x, pt.y, pt.x, pt.y);

	for (int i=1; i < m_arPoint.GetSize(); i++)
	{
		pt = m_arPoint[i];
		m_rcBound.left     = min(m_rcBound.left, pt.x);
		m_rcBound.right    = max(m_rcBound.right, pt.x);
		m_rcBound.top      = max(m_rcBound.top, pt.y);
		m_rcBound.bottom   = min(m_rcBound.bottom, pt.y);
	}

	m_rcBound.InflateRect(CSize(PEN_WIDTH, -(int)PEN_WIDTH));
	return;
}

CStrokeList::CStrokeList()
{

}

CStrokeList::~CStrokeList()
{
	Clear();
}

void CStrokeList::Draw(CDC* pDC) const
{
	int nCount = m_lstStroke.GetCount();
	POSITION posStrokeList = m_lstStroke.GetHeadPosition();
	CStroke* pStroke;
	for (int i = 0; i < nCount; i++)
	{
		pStroke = m_lstStroke.GetNext(posStrokeList);
		pStroke->Draw(pDC);
	}
}

void CStrokeList::Clear()
{
	int nCount = m_lstStroke.GetCount();
	
	POSITION posStrokeList = m_lstStroke.GetHeadPosition();
	CStroke* pStroke;
	for (int i = 0; i < nCount; i++)
	{
		pStroke = m_lstStroke.GetNext(posStrokeList);
		if (pStroke!= NULL)
		{
			delete pStroke;
			pStroke = NULL;
		}
	}
	m_lstStroke.RemoveAll();
}

CRect CStrokeList::Undo()
{
	CRect rcUpdate;
	rcUpdate.SetRectEmpty();
	int nCount = m_lstStroke.GetCount();
	if (nCount > 0)
	{
		POSITION pos = m_lstStroke.GetTailPosition();
		CStroke* pStroke = m_lstStroke.GetAt(pos);
		rcUpdate = pStroke->GetBoundRect();
		if (pStroke != NULL)
		{
			delete pStroke;
			pStroke = NULL;
		}
		m_lstStroke.RemoveTail();
	}
	return rcUpdate;
}

void CStrokeList::StartStroke()
{
	CStroke* pStroke = new CStroke();
	
	m_lstStroke.AddTail(pStroke);
}

void CStrokeList::EndStroke()
{
	POSITION pos = m_lstStroke.GetTailPosition();
	if (pos != NULL)
	{
		CStroke* pStroke = m_lstStroke.GetAt(pos);
		if ( pStroke->GetNumStroke() > 1) //ϳ ִ   ʴ´.
			pStroke->Finish();
		else
		{
			delete pStroke;
			m_lstStroke.RemoveTail();
		}
	}
}

void CStrokeList::AddPoint(CPoint* pPoint)
{
	POSITION pos = m_lstStroke.GetTailPosition();
	if (pos != NULL)
	{
		CStroke* pStroke = m_lstStroke.GetAt(pos);
		pStroke->AddPoint(pPoint);
	}	
}
