// StrokeView.cpp : implementation file
//
#include "stdafx.h"
#include "resource.h"
#include "StrokeView.h"
#include "Stroke.h"
#include "define.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const int CStrokeView::BTN_WIDTH = 20;
const int CStrokeView::DEFAULT_PEN_WIDTH = 2;

#define ID_CLEAR_STROKE 51001
#define ID_UNDO_STROKE  51002

/////////////////////////////////////////////////////////////////////////////
// CStrokeView

CStrokeView::CStrokeView()
{
	m_pStrokeList = new CStrokeList();
	m_pWnd = NULL;
	m_rgbPen = RGB(0, 0, 0);
	m_nPenWidth = DEFAULT_PEN_WIDTH;
}

CStrokeView::~CStrokeView()
{
	if (m_pStrokeList != NULL)
	{
		delete m_pStrokeList;
		m_pStrokeList = NULL;
	}
}


/////////////////////////////////////////////////////////////////////////////
// CStrokeView message handlers

int CStrokeView::OnCreate(CWnd* pWnd) 
{
	m_pWnd = pWnd;

	RebuildPen();
	
	CRect rcButton(0, 0, 0, 0);

	m_btnClear.Create(_T(""), 
		WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON | BS_CENTER | BS_VCENTER , 
		rcButton, pWnd, ID_CLEAR_STROKE);
	m_btnClear.SetIcon(IDI_CLEAR, CSize(16,16));

	m_btnUndo.Create(_T(""), 
		WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON | BS_CENTER | BS_VCENTER , 
		rcButton, pWnd, ID_UNDO_STROKE);
	m_btnUndo.SetIcon(IDI_UNDO, CSize(16,16));
	
	return 0;
}

void CStrokeView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (m_bShow == FALSE)
		return;

	m_pStrokeList->StartStroke();
	m_pStrokeList->AddPoint(&point);
	m_ptPrev = point;
	
	m_pWnd->SetCapture();
	//CWnd::OnLButtonDown(nFlags, point);
}

void CStrokeView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_bShow == FALSE)
		return;

	m_pStrokeList->AddPoint(&point);
	m_pStrokeList->EndStroke();

	ReleaseCapture();
	
	//CWnd::OnLButtonUp(nFlags, point);
}

void CStrokeView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_bShow == FALSE)
		return;

	if (m_pWnd->GetCapture() != m_pWnd)
		return; // If this window (view) didn't capture the mouse,

	m_pStrokeList->AddPoint(&point);

	CClientDC dc(m_pWnd);

	CPen* pOldPen = dc.SelectObject(&m_penDraw);
	dc.MoveTo(m_ptPrev);
	dc.LineTo(point);
	dc.SelectObject(pOldPen);
	m_ptPrev = point;
}


void CStrokeView::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	switch(LOWORD(wParam))
	{
		case ID_CLEAR_STROKE:
			m_pStrokeList->Clear();
			m_pWnd->InvalidateRect(NULL, TRUE);
			break;
		case ID_UNDO_STROKE:
		{
			CRect rcUpdate = m_pStrokeList->Undo();
			m_pWnd->InvalidateRect(rcUpdate, TRUE);
			break;
		}
	}
}

void CStrokeView::OnPaint(CDC* pDC) 
{
	if (m_bShow == TRUE)
	{
		CPen* pPenOld = pDC->SelectObject(&m_penDraw);
		m_pStrokeList->Draw(pDC);
		pDC->SelectObject(pPenOld);
	}
}

void CStrokeView::Clear()
{
	m_pStrokeList->Clear();
	m_pWnd->InvalidateRect(NULL, TRUE);
}

void CStrokeView::OnSize(UINT nType, int cx, int cy, DWORD dwButtonSize) 
{
	CRect rcClient;
	//m_pWnd->GetClientRect(rcClient);

	rcClient.left = 4;
	rcClient.right = cx - 4;
	rcClient.bottom = (cy - 4) - dwButtonSize - 4;
	rcClient.top = rcClient.bottom - STROKE_VIEW_HEIGHT;

	CRect rcButton;
	rcButton.right = cx - 2;
	rcButton.left = rcButton.right - BTN_WIDTH;
	rcButton.top = rcClient.top + 2;
	rcButton.bottom = rcButton.top + (rcClient.Height() - 2 * 3 ) / 2;

	m_btnClear.MoveWindow(&rcButton, TRUE);

	rcButton.top = rcClient.top + 2 + (rcClient.Height() - 2 * 3 ) / 2 + 2;
	rcButton.bottom = rcButton.top + (rcClient.Height() - 2 * 3 ) / 2;

	m_btnUndo.MoveWindow(&rcButton, TRUE);

}

void CStrokeView::ShowView(BOOL bShow)
{
	m_bShow = bShow;
	if (m_bShow == TRUE)
	{
		m_btnClear.ShowWindow(SW_SHOW);
		m_btnUndo.ShowWindow(SW_SHOW);
	}
	else
	{
		m_btnClear.ShowWindow(SW_HIDE);
		m_btnUndo.ShowWindow(SW_HIDE);
	}
}

void CStrokeView::SetPenColor(COLORREF rgbPen)
{  
	if (m_rgbPen != rgbPen)
	{
		m_rgbPen = rgbPen; 
		RebuildPen();
	}
}

void CStrokeView::SetPenWidth(int nPenWidth)
{
	if (m_nPenWidth != nPenWidth)
	{
		m_nPenWidth = nPenWidth;
		RebuildPen();
	}
}


void CStrokeView::RebuildPen()
{
	if (m_penDraw.m_hObject != NULL)
		m_penDraw.DeleteObject();
	m_penDraw.CreatePen(PS_SOLID, m_nPenWidth, m_rgbPen);
}