// Template.cpp: implementation of the CTemplate class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "define.h"
#include "MemoKing.h"
#include "Template.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTemplate::CTemplate()
{
	for (int i = 0; i < MAX_ITEM; i++)
	{
		wsprintf(m_arItem[i].szName, _T("Item%d"), i);
		m_arItem[i].bShowInQuestion = true;
		m_arItem[i].bShowInAnswer = true;
		m_arItem[i].bUseClearType = DEF_CLEARTYPE;
		m_arItem[i].bBold = DEF_BOLD; 
		m_arItem[i].bItalic = DEF_ITALIC;
		m_arItem[i].nFontSize = DEF_SIZE;
		m_arItem[i].nAlign = DT_LEFT;
		m_arItem[i].crFont = RGB(0, 0, 0);		
		m_arItemEx[i].dwCharSet = 0;
		lstrcpy(m_arItemEx[i].szFontName, DEFAULT_FONT);
		m_arItemEx[i].dwReserved = 0;
	}
	m_bModified = false;
}

CTemplate::~CTemplate()
{

}

void CTemplate::SetFontName(UINT uiIndex, LPCTSTR szName, DWORD dwCharSet)
{
	ASSERT(uiIndex < MAX_ITEM);
	ASSERT(lstrlen(szName) < LF_FACESIZE);

	lstrcpy(m_arItemEx[uiIndex].szFontName, szName);
	m_arItemEx[uiIndex].dwCharSet = dwCharSet;
	m_bModified = true;
}

void CTemplate::SetItemName(UINT uiIndex, LPCTSTR szName)
{
	ASSERT(uiIndex < MAX_ITEM);
	ASSERT(lstrlen(szName) < MAX_ITEM_NAME);

	lstrcpy(m_arItem[uiIndex].szName, szName);
	m_bModified = true;
}


bool CTemplate::ExportToFile(LPCTSTR szFileName) const
{
	ASSERT(szFileName != NULL);

	CFile fileTemplate;
	fileTemplate.Open( szFileName, CFile::modeCreate | CFile::modeWrite) ;

	TEMPLATE_HEADER Header;
	Header.wType = MKT_FILE_TYPE;
	Header.wVersion = MK_FILE_VER;
	fileTemplate.Write((void*)&Header, sizeof(TEMPLATE_HEADER));

	SaveToFile(&fileTemplate);

	fileTemplate.Close();

	return true;
}

bool CTemplate::ImportFromFile(LPCTSTR szFileName)
{
	ASSERT(szFileName != NULL);

	CFile fileTemplate;
	fileTemplate.Open( szFileName, CFile::modeRead);

	TEMPLATE_HEADER Header;
	fileTemplate.Read((void*)&Header, sizeof(TEMPLATE_HEADER));

	if (Header.wType != MKT_FILE_TYPE)
	{
		//  Ʋ 
		return false;
	}

	if (Header.wVersion > MK_FILE_VER) // α׷ ϴ  ū  
	{
		return false;
	}

	LoadFromFile(&fileTemplate, Header.wVersion);

	fileTemplate.Close();
	m_bModified = true;

	return FALSE;
}


bool CTemplate::SaveToFile(CFile* pFile) const
{
	ASSERT(pFile != NULL);

	for (int i = 0; i < MAX_ITEM; i++)
	{
		pFile->Write((void*)&m_arItem[i], sizeof(ITEM));
		pFile->Write((void*)&m_arItemEx[i], sizeof(ITEM_EX));
	}

	return true;
}

bool CTemplate::SaveToFile(HANDLE hFile) const
{
	ASSERT(hFile != NULL);
	
	DWORD dwWritten;
	for (int i = 0; i < MAX_ITEM; i++)
	{
		WriteFile(hFile, (void*)&m_arItem[i], sizeof(ITEM), &dwWritten, NULL);
		WriteFile(hFile, (void*)&m_arItemEx[i], sizeof(ITEM_EX), &dwWritten, NULL);
	}
	return true;
}

bool CTemplate::LoadFromFile(CFile* pFile, WORD wVersion)
{
	ASSERT(pFile != NULL);

	DWORD dwRead;
	if (wVersion < 0x0003) // Ver3     
	{

		for (int i = 0; i < MAX_ITEM; i++)
		{
			dwRead = pFile->Read((void*)&m_arItem[i], sizeof(ITEM));
			if (dwRead != sizeof(ITEM))
				return false;

			m_arItemEx[i].dwCharSet = ANSI_CHARSET;
			lstrcpy(m_arItemEx[i].szFontName, DEFAULT_FONT);
			m_arItemEx[i].dwReserved = 0;
		}

	}
	else
	{
		for (int i = 0; i < MAX_ITEM; i++)
		{
			dwRead = pFile->Read((void*)&m_arItem[i], sizeof(ITEM));
			if (dwRead != sizeof(ITEM))
				return false;
			dwRead = pFile->Read((void*)&m_arItemEx[i], sizeof(ITEM_EX));
			if (dwRead != sizeof(ITEM_EX))
				return false;
		}
	}

	return true;
}

bool CTemplate::IsMixedCharSet() const
{
	DWORD dwCharSet = m_arItemEx[0].dwCharSet;

	for (int i = 1; i < MAX_ITEM; i++)
	{
		if ( dwCharSet != m_arItemEx[i].dwCharSet)
			return true;
	}

	return false;
}