// Template.h: interface for the CTemplate class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TEMPLATE_H__F1D0A471_50E7_4BA0_9E82_7AD598412E4D__INCLUDED_)
#define AFX_TEMPLATE_H__F1D0A471_50E7_4BA0_9E82_7AD598412E4D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef struct _TEMPLATE_HEADER
{
	WORD wType;
	WORD wVersion;
} TEMPLATE_HEADER;

typedef struct _ITEM
{
	TCHAR szName[MAX_ITEM_NAME]; 
	BOOL bShowInQuestion;
	BOOL bShowInAnswer; 
	BOOL bUseClearType; 
	BOOL bBold;           
	BOOL bItalic;
	UINT nFontSize;
	UINT nAlign;
	UINT nUpperMargin;
	COLORREF crFont;
} ITEM;

typedef struct _ITEM_EX
{
	TCHAR szFontName[LF_FACESIZE];
	DWORD dwCharSet;
	DWORD dwReserved;
} ITEM_EX;


class CTemplate
{
private:
	ITEM m_arItem[MAX_ITEM];
	ITEM_EX m_arItemEx[MAX_ITEM];
	bool m_bModified;
public:

	CTemplate();
	virtual ~CTemplate();
	bool IsModified() const { return m_bModified; }

	void SetFontName(UINT uiIndex, LPCTSTR szFontName, DWORD dwCharSet);
	LPCTSTR GetFontName(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItemEx[uiIndex].szFontName;
	}
	BYTE GetFontCharSet(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return (BYTE)(m_arItemEx[uiIndex].dwCharSet);
	}

	void SetItemName(UINT uiIndex, LPCTSTR szName);
	LPCTSTR GetItemName(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].szName;
	}

	void SetShowInQuestion(UINT uiIndex, BOOL bShow)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].bShowInQuestion = bShow;
		m_bModified = true;
	}
	BOOL IsShowInQuestion(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].bShowInQuestion;
	}

	void SetShowInAnswer(UINT uiIndex, BOOL bShow)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].bShowInAnswer = bShow;
		m_bModified = true;
	}
	BOOL IsShowInAnswer(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].bShowInAnswer;
	}

	void SetBoldFont(UINT uiIndex, BOOL bBold)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].bBold = bBold;
		m_bModified = true;
	}
	BOOL IsBoldFont(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].bBold;
	}

	void SetItalicFont(UINT uiIndex, BOOL bItalic)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].bItalic = bItalic;
		m_bModified = true;
	}
	BOOL IsItalicFont(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].bItalic;
	}

	void SetClearTypeFont(UINT uiIndex, BOOL bClearType)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].bUseClearType = bClearType;
		m_bModified = true;
	}
	BOOL IsClearTypeFont(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].bUseClearType ;
	}

	void SetFontSize(UINT uiIndex, UINT nFontSize)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].nFontSize = nFontSize;
		m_bModified = true;
	}
	UINT GetFontSize(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].nFontSize;
	}

	void SetAlign(UINT uiIndex, UINT uiAlign)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].nAlign = uiAlign;
		m_bModified = true;
	}
	UINT GetAlign(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].nAlign;
	}

	void SetUpperMargin(UINT uiIndex, UINT uiMargin)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].nUpperMargin = uiMargin;
		m_bModified = true;
	}
	UINT GetUpperMargin(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].nUpperMargin;
	}

	void SetFontColor(UINT uiIndex, COLORREF crFont)
	{
		ASSERT(uiIndex < MAX_ITEM);
		m_arItem[uiIndex].crFont = crFont;
		m_bModified = true;
	}
	COLORREF GetFontColor(UINT uiIndex) const
	{
		ASSERT(uiIndex < MAX_ITEM);
		return m_arItem[uiIndex].crFont;
	}

	bool ExportToFile(LPCTSTR szFileName) const;
	bool ImportFromFile(LPCTSTR szFileNmae);
	bool SaveToFile(CFile* pFile) const;
	bool SaveToFile(HANDLE hFile) const;
	bool LoadFromFile(CFile* pFile, WORD wVersion);
	bool IsMixedCharSet() const;
};

#endif // !defined(AFX_TEMPLATE_H__F1D0A471_50E7_4BA0_9E82_7AD598412E4D__INCLUDED_)

