// QuizSetup.cpp : implementation file
//
#include "stdafx.h"
#include "resource.h"
#include "define.h"
#include "QuizSetupDlg.h"
#include "Quiz.h"
#include "Template.h"
#include "Utils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CQuizSetupDlg dialog


CQuizSetupDlg::CQuizSetupDlg(CQuizSetting* pQuiz, CTemplate* pTemplate, UINT uiMaxCards, CWnd* pParent /*=NULL*/)
	: CDialog(CQuizSetupDlg::IDD, pParent)
{
	ASSERT(pTemplate != NULL);
	//{{AFX_DATA_INIT(CQuizSetupDlg)
	m_nNumQuiz = 0;
	m_uiFontSize = 0;
	m_uiEndCardNum = 0;
	m_uiStartCardNum = 0;
	m_bInternalExam = FALSE;
	m_bQuizResultSync = FALSE;
	//}}AFX_DATA_INIT

	m_pQuizSetting = pQuiz;

	m_uiMaxCards = uiMaxCards;
	m_pTemplate = pTemplate;

	m_bRebuildFont = FALSE;
}


void CQuizSetupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CQuizSetupDlg)
	DDX_Control(pDX, IDC_NUM_EXAMPLE, m_wndNumExample);
	DDX_Control(pDX, IDC_QUESTION, m_wndQuestion);
	DDX_Control(pDX, IDC_ANSWER, m_wndAnswer);
	DDX_Text(pDX, IDC_NUM_QUIZ, m_nNumQuiz);
	DDX_Text(pDX, IDC_FONT_SIZE, m_uiFontSize);
	DDX_Text(pDX, IDC_END_CARD_NUM, m_uiEndCardNum);
	DDX_Text(pDX, IDC_START_CARD_NUM, m_uiStartCardNum);
	DDX_Check(pDX, IDC_INTERNAL_EXAM_MODE, m_bInternalExam);
	DDX_Check(pDX, IDC_QUIZ_RESULT_SYNC, m_bQuizResultSync);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CQuizSetupDlg, CDialog)
	//{{AFX_MSG_MAP(CQuizSetupDlg)
	ON_BN_CLICKED(IDC_QUIZ_10, OnQuiz10)
	ON_BN_CLICKED(IDC_QUIZ_20, OnQuiz20)
	ON_BN_CLICKED(IDC_QUIZ_40, OnQuiz40)
	ON_BN_CLICKED(IDC_WHOLE_CARDS, OnWholeCards)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_INTERNAL_EXAM_MODE, OnInternalExamMode)
	ON_BN_CLICKED(IDC_QUIZ_RESULT_SYNC, OnQuizResultSync)
	ON_BN_CLICKED(IDC_FROM_ALL_CARDS, OnFromAllCards)
	ON_BN_CLICKED(IDC_FROM_MEMORIZED_CARDS, OnFromMemorizedCards)
	ON_BN_CLICKED(IDC_FROM_NOTMEMORIZED_CARDS, OnFromNotMemorizedCards)
	ON_BN_CLICKED(IDC_FROM_SELECTED_CARDS, OnFromSelectedCards)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CQuizSetupDlg::OnOK()
{
	UpdateData(TRUE);
	//˻
	if (m_nNumQuiz <= 0)
	{
		CString strMsg;
		strMsg.LoadString(IDS_WRONG_QUIZ_NUMBER);
		MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
		return;
	}

	if (m_nNumQuiz > m_uiMaxCards)
	{
		CString strMsg;
		strMsg.LoadString(IDS_TOO_MANY_QUIZ_NUMBER);
		MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
		return;
	}

	if (m_wndQuestion.GetCurSel() == m_wndAnswer.GetCurSel())
	{
		CString strMsg;
		strMsg.LoadString(IDS_SAME_ITEM_SELECT);
		MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
		return;
	}

	int nNumExam = m_wndNumExample.GetCurSel() + NUM_MIN_EXAMPLE;
	if (nNumExam < NUM_MIN_EXAMPLE || nNumExam > NUM_MAX_EXAMPLE)
	{
		CString strMsg;
		strMsg.LoadString(IDS_INVALID_NUM_EXAM);
		MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
		return;
	}
	
	m_pQuizSetting->SetInternalExamMode(m_bInternalExam);
	m_pQuizSetting->SetQuizResultSync(m_bQuizResultSync);

	m_pQuizSetting->SetTotalQuiz(m_nNumQuiz);
	m_pQuizSetting->SetQuestionIndex(m_wndQuestion.GetCurSel());

	if (m_pQuizSetting->GetAnswerIndex() != m_wndAnswer.GetCurSel())
	{
		m_pQuizSetting->SetAnswerIndex(m_wndAnswer.GetCurSel());
		m_bRebuildFont = TRUE;
	}

	m_pQuizSetting->SetNumExample(m_wndNumExample.GetCurSel() + NUM_MIN_EXAMPLE);
	if (m_pQuizSetting->GetFontSize() != m_uiFontSize)
	{
		m_bRebuildFont = TRUE;
		m_pQuizSetting->SetFontSize(m_uiFontSize);
	}

	CButton* pButton;
	pButton = (CButton*)GetDlgItem(IDC_FROM_MEMORIZED_CARDS);
	if (pButton->GetCheck() == BST_CHECKED)
		m_pQuizSetting->SetMemorizedCardOnly(TRUE);
	else
		m_pQuizSetting->SetMemorizedCardOnly(FALSE);

	/* [ȯ]  */
	pButton = (CButton*)GetDlgItem(IDC_FROM_NOTMEMORIZED_CARDS);
	if (pButton->GetCheck() == BST_CHECKED)
		m_pQuizSetting->SetNotMemorizedCardOnly(TRUE);
	else
		m_pQuizSetting->SetNotMemorizedCardOnly(FALSE);
	/* [ȯ]  */

	pButton = (CButton*)GetDlgItem(IDC_FROM_SELECTED_CARDS);
	if (pButton->GetCheck() == BST_CHECKED)
	{
		// ˻ 
		if (m_uiStartCardNum < 1 || m_uiEndCardNum > m_uiMaxCards)
		{
			CString strMsg;
			strMsg.LoadString(IDS_INVALID_QUIZ_RANGE);
			MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
			return;
		}

		if (m_uiStartCardNum >=m_uiEndCardNum)
		{
			CString strMsg;
			strMsg.LoadString(IDS_INVALID_QUIZ_RANGE);
			MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
			return;
		}

		if (m_uiEndCardNum - m_uiStartCardNum + 1 < m_nNumQuiz)
		{
			CString strMsg;
			strMsg.LoadString(IDS_TOO_SMALL_QUIZ_RANGE);
			MessageBox(strMsg, AfxGetAppName(), MB_OK | MB_ICONWARNING);
			return;
		}

		m_pQuizSetting->SetSelectedCardOnly(TRUE);
		m_pQuizSetting->SetStartCardIndex(m_uiStartCardNum-1);
		m_pQuizSetting->SetEndCardIndex(m_uiEndCardNum-1);
	}
	else
		m_pQuizSetting->SetSelectedCardOnly(FALSE);


	CDialog::OnOK();
}
/////////////////////////////////////////////////////////////////////////////
// CQuizSetupDlg message handlers

void CQuizSetupDlg::OnQuiz10() 
{
	m_nNumQuiz = 10;
	UpdateData(FALSE);
}

void CQuizSetupDlg::OnQuiz20() 
{
	m_nNumQuiz = 20;
	UpdateData(FALSE);
}

void CQuizSetupDlg::OnQuiz40() 
{
	m_nNumQuiz = 40;
	UpdateData(FALSE);
}

void CQuizSetupDlg::OnWholeCards() 
{
	m_nNumQuiz = m_uiMaxCards;
	UpdateData(FALSE);
}

BOOL CQuizSetupDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_bInternalExam = m_pQuizSetting->IsInternalExamMode();
	m_bQuizResultSync = m_pQuizSetting->IsQuizResultSync();
	GetDlgItem(IDC_ANSWER)->EnableWindow(m_bInternalExam != TRUE);

	m_nNumQuiz = m_pQuizSetting->GetTotalQuiz();
	if (m_nNumQuiz > m_uiMaxCards)
		m_nNumQuiz = m_uiMaxCards;
	m_uiFontSize = m_pQuizSetting->GetFontSize();
	m_uiStartCardNum = m_pQuizSetting->GetStartCardIndex() + 1;
	m_uiEndCardNum = m_pQuizSetting->GetEndCardIndex() + 1;
	SendDlgItemMessage(IDC_START_CARD_NUM_SPIN,UDM_SETRANGE,0,MAKELONG(m_uiMaxCards, 1));
	SendDlgItemMessage(IDC_END_CARD_NUM_SPIN,UDM_SETRANGE,0,MAKELONG(m_uiMaxCards, 1));

	//List ߰ 
	int i;
	for (i = 0; i < MAX_ITEM; i++)
	{
		m_wndQuestion.InsertString(i, m_pTemplate->GetItemName(i));
	}
	m_wndQuestion.SetCurSel(m_pQuizSetting->GetQuestionIndex());

	for (i = 0; i < MAX_ITEM; i++)
	{
		m_wndAnswer.InsertString(i, m_pTemplate->GetItemName(i));
	}
	m_wndAnswer.SetCurSel(m_pQuizSetting->GetAnswerIndex());

	SendDlgItemMessage(IDC_NUM_QUIZ_SPIN,UDM_SETRANGE,0,MAKELONG(m_uiMaxCards, 1));
	SendDlgItemMessage(IDC_QUIZ_FONT_SPIN,UDM_SETRANGE,0,MAKELONG(50, 7));

	m_wndNumExample.SetCurSel(m_pQuizSetting->GetNumExample() - NUM_MIN_EXAMPLE);

	CButton* pButton;
	if (m_pQuizSetting->IsMemorizedCardOnly() == TRUE)
	{
		pButton = (CButton*)GetDlgItem(IDC_FROM_MEMORIZED_CARDS);
		pButton->SetCheck(BST_CHECKED);
	}
	else if (m_pQuizSetting->IsNotMemorizedCardOnly() == TRUE)
	{
		pButton = (CButton*)GetDlgItem(IDC_FROM_NOTMEMORIZED_CARDS);
		pButton->SetCheck(BST_CHECKED);
	}
	else if (m_pQuizSetting->IsSelectedCardOnly() == TRUE)
	{
		pButton = (CButton*)GetDlgItem(IDC_FROM_SELECTED_CARDS);
		pButton->SetCheck(BST_CHECKED);
	}
	else
	{
		pButton = (CButton*)GetDlgItem(IDC_FROM_ALL_CARDS);
		pButton->SetCheck(BST_CHECKED);
	}
	EableQuizRangeControls(m_pQuizSetting->IsSelectedCardOnly());

	UpdateData(FALSE);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CQuizSetupDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CString strMsg;
	strMsg.LoadString(IDS_QUIZ_SETTING);
	::PaintHeader(dc, strMsg);
}


void CQuizSetupDlg::OnInternalExamMode() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_ANSWER)->EnableWindow(m_bInternalExam != TRUE);
}

/*********** [ȯ] .  **************/
void CQuizSetupDlg::OnQuizResultSync() 
{
	UpdateData(TRUE);
//	GetDlgItem(IDC_ANSWER)->EnableWindow(m_bQuizResultSync != TRUE);
}
/*********** [ȯ] .  **************/

void CQuizSetupDlg::OnFromAllCards() 
{
	EableQuizRangeControls(FALSE);
}

void CQuizSetupDlg::OnFromMemorizedCards() 
{
	EableQuizRangeControls(FALSE);
}

void CQuizSetupDlg::OnFromNotMemorizedCards() 
{
	EableQuizRangeControls(FALSE);
}

void CQuizSetupDlg::OnFromSelectedCards() 
{
	EableQuizRangeControls(TRUE);
}

void CQuizSetupDlg::EableQuizRangeControls(BOOL bEnable)
{
	GetDlgItem(IDC_START_CARD_NUM)->EnableWindow(bEnable);
	GetDlgItem(IDC_END_CARD_NUM)->EnableWindow(bEnable);	
	GetDlgItem(IDC_START_CARD_NUM_SPIN)->EnableWindow(bEnable);
	GetDlgItem(IDC_END_CARD_NUM_SPIN)->EnableWindow(bEnable);
}
